﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WpfApplication1
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {

        /// <summary>
        /// 読み込んだ画像を保存するための変数
        /// </summary>
        BitmapImage bmpImage;

        /// <summary>
        /// 画像を書き込み編集するための変数
        /// </summary>
        WriteableBitmap writableBmp;

        public MainWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Loadボタンを押したときの処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_Load_Click(object sender, RoutedEventArgs e)
        {
            // ファイルを開くダイアログ
            Microsoft.Win32.OpenFileDialog ofDialog =
                new Microsoft.Win32.OpenFileDialog();

            if ((bool)ofDialog.ShowDialog())
            {
                //ファイルからBitmapImageにデータを読み込む
                bmpImage = new BitmapImage(new Uri(ofDialog.FileName));

                //編集する準備
                writableBmp = new WriteableBitmap(bmpImage);

                //画像を表示する(編集する画像データにする)
                image1.Source = writableBmp;
            }
        }

        /// <summary>
        /// ToRedボタンを押したときの処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_ToRed_Click(object sender, RoutedEventArgs e)
        {
            //データを保存する先を用意する
            byte[] dataArray = new byte[writableBmp.PixelHeight * writableBmp.BackBufferStride];

            //ピクセルのデータを全て保存
            writableBmp.CopyPixels(dataArray, writableBmp.BackBufferStride, 0);

            //保存したデータは 1pixel = {[B][G][R][X]} の並びで保存されている。
            //各値は0～255。ここではBとGを0にして、Rの成分だけそのままにする。
            for (int i = 0; i < dataArray.Length; i += 4)
            {
                dataArray[i] = 0;//B
                dataArray[i + 1] = 0;//G
                //dataArray[i + 2] = 0;//R
                //dataArray[i + 3] = 0;//余剰分の値
            }

            //Bitmapに書き込み
            writableBmp.WritePixels(
                  //書き込む範囲(x=0,y=0の位置から,画像の幅と高さ分の範囲を書き換える)
                  new Int32Rect(0, 0, writableBmp.PixelWidth, writableBmp.PixelHeight),
                  //書き換えるデータ(先に編集した配列)
                  dataArray,
                  //書き換えるストライド
                  writableBmp.BackBufferStride,
                  0);
        }

        /// <summary>
        /// ToGrayボタンを押したときの処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_ToGray_Click(object sender, RoutedEventArgs e)
        {
            //データを保存する先を用意する
            byte[] dataArray = new byte[writableBmp.PixelHeight * writableBmp.BackBufferStride];

            //ピクセルのデータを全て保存
            writableBmp.CopyPixels(dataArray, writableBmp.BackBufferStride, 0);

            //灰色の値
            int grayColor = 0;

            //保存したデータは 1pixel = {[B][G][R][X]} の並びで保存されている。
            //各値は0～255。ここではBとGを0にして、Rの成分だけそのままにする。
            for (int i = 0; i < dataArray.Length; i += 4)
            {
                //BGR成分の値をそれぞれ合算する
                grayColor += dataArray[i];
                grayColor += dataArray[i + 1];
                grayColor += dataArray[i + 2];
                //合算した後3で割って平均をとる(※byteは0～255の値しか取れないので注意)
                grayColor /= 3;

                //求めた値をRGBの各成分にする
                dataArray[i] = (byte)grayColor;
                dataArray[i + 1] = (byte)grayColor;
                dataArray[i + 2] = (byte)grayColor;

                //灰色の値を初期化
                grayColor = 0;
            }

            //Bitmapに書き込み
            writableBmp.WritePixels(
                //書き込む範囲(x=0,y=0の位置から,画像の幅と高さ分の範囲を書き換える)
                  new Int32Rect(0, 0, writableBmp.PixelWidth, writableBmp.PixelHeight),
                //書き換えるデータ(先に編集した配列)
                  dataArray,
                //書き換えるストライド
                  writableBmp.BackBufferStride,
                  0);
        }

    }
}
